/*! 
*   @file cook.h overall program control header file
*   @author Adam Mendrela, Andrew Kerns
*/

#ifndef __COOK_H_GUARD_
#define __COOK_H_GUARD_

#ifndef F_CPU
	#warning "define F_CPU in project config"
	#define F_CPU 16000000UL
#endif
#include <util/delay.h>
#include <avr/io.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h> 
#include <inttypes.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h> //Required for PROGMEM use
#include "uart.h"
#include "stepper.h"
#include "userinterface.h"
#include "lcd_lib.h" 
#include "readNTC.h"
#include "readIC.h"

#ifdef DEBUG
	#include "uart.h"
	FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);
#endif

#define FORC_C					0
#define FORC_F					1

#define MODE_WELCOME			1
#define MODE_WARNING			2
#define MODE_HOME				3
#define MODE_AUTOTEMP			4
#define MODE_DCHEAT1			5
#define MODE_DCHEAT2			13
#define MODE_AUTOCOOK 			6
#define MODE_COOK				9
#define MODE_CAL1				10
#define MODE_CAL2				11	
#define MODE_CAL3				12

#define PROBE_NTC				0
#define PROBE_IC				1

#define WAIT_MAIN_MS			500
#define WAIT_KEY_MS				22
#define WAIT_MEAS_MS			20

#define NUM_MEAS 				2

#define EEPROM_CHECKVAL  		0x22  //EEPROM[50:51] will each equal 0x22 after EEPROM has been written
#define EEPROM_BYTE_CHECK0  	50
#define EEPROM_BYTE_CHECK1  	51
#define EEPROM_BYTE_MAXPOS 		52
#define EEPROM_BYTE_ONPOS		53
#define MAXPOS_DEFAULT 			41   // 41 is good for hornilla (hot plate).  for demo of cal mode, make is 30 or something
#define ONPOS_DEFAULT			10

#define QMAX					750

// externally visible variables
uint8_t mode;
uint8_t ForC;
int8_t sstate;
int8_t stepper_maxpos;
int8_t stepper_onpos;
int8_t max_sstate;
uint8_t probe;
uint8_t timer_s, timer_min, timer_done, timer_buzzing, timer_on;

float error; //single error 
float preverror; 
float accerror;
float currT;
float T;
float Td;
float Td_deadzone;
float Q;

// externally visible functions
void selectProbe(void);
void softReset(void);
void setTimerMinSec(uint8_t min, uint8_t sec);

#endif
