/*! 
*   @file readNTC.c NTC thermistor measurement code header file
*   @author Adam Mendrela, Andrew Kerns
*/

/**
% checking calcs in Matlab
T = whatever
R0 = 10e3;
T0 = 298.15; % 25C
B = 3400;
Rs = 5e3; % the resistor between Vcc and the NTC
rNTC = R0*exp(B*(1./T - 1/T0))
vNTC = [rNTC/(Rs+rNTC)] * 5
LSBs = 1024*(vNTC/5)
% now go in reverse
vNTC = [LSBs/1024]*5
rNTC = Rs*vNTC/(1-vNTC)
T = B/(ln(rNTC) - ln(R0) + B/T0)
**/

// 230 LSBs is 4.5098 V
// 26 LSBs is 

/*

#ifndef F_CPU
	#define F_CPU 16000000UL
#endif

#include <stdio.h>
#include <stdlib.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <math.h>

#include "uart.h"

FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

*/

// ------------ REAL START TO FILE -------------------

#include "readNTC.h"

uint8_t NTCmode;
float T_NTC;   //calculated temperature	
float vNTCby5; //voltage measured
float xfloat;
//#ifdef DEBUG
	char v_string[9];
	char g_string[9];
//#endif
uint16_t x, AinHigh, AinLow;

inline void set_Ain_A0 ()
{
	ADMUX &= ~(1<<MUX0);
}

float readNTC()
{
	
	//fprintf(stdout, "reading ntc");
   	AinLow = (uint16_t)(ADCL >> 6);
   	AinHigh = ((uint16_t)ADCH) << 2; 
   	x = AinHigh + AinLow;

	//#if DEBUG
		fprintf(stdout, "x = %u, AinLow = %u, AinHigh = %u \n\r", x, AinLow, AinHigh);
	//#endif
	xfloat = (5.0*(float)x)/1023.0;
	//#if DEBUG
	dtostrf(xfloat, 6, 3, v_string);
	fprintf(stdout, "x = %s V, mode = %u \n\r",v_string, NTCmode);
	//#endif
	
	//Calculate new calculated voltage
	switch(NTCmode)
	{ // vNTCby5 = vNTC/5
		case 1:  vNTCby5 = (1/NTC_MODE1_GAIN)*((float)x)/1024.0; 
				#if DEBUG
					dtostrf(NTC_MODE1_GAIN, 6, 3, g_string); 
				#endif
				break;
		case 2:  vNTCby5 = (1/NTC_MODE2_GAIN)*((float)x)/1024.0; 
				#if DEBUG
					dtostrf(NTC_MODE2_GAIN, 6, 3, g_string); 
				#endif
				break;
		case 3:  vNTCby5 = (1/NTC_MODE3_GAIN)*((float)x)/1024.0; 
				#if DEBUG
					dtostrf(NTC_MODE3_GAIN, 6, 3, g_string); 
				#endif
				break;
		case 4:  vNTCby5 = (1/NTC_MODE4_GAIN)*((float)x)/1024.0; 
				#if DEBUG
					dtostrf(NTC_MODE4_GAIN, 6, 3, g_string); 
				#endif
				break;
		case 5:  vNTCby5 = (1/NTC_MODE5_GAIN)*((float)x)/1024.0; 
				#if DEBUG
					dtostrf(NTC_MODE5_GAIN, 6, 3, g_string); 
				#endif
				break;
		default: vNTCby5 = 0.001;
	}

	#if DEBUG
	    dtostrf(vNTCby5*5.0, 6, 3, v_string);
		fprintf(stdout, "vNTC = %s gain = %s \n\r", v_string,g_string);
	#endif
	
	T_NTC = B/ (log(RS*(vNTCby5)/(1-(vNTCby5))) - LOG_R0 + B_BY_T0);
	
	//autoranging
	if (xfloat >= 4.75)   
	{
		if (NTCmode > 1)
			NTCmode--;
		//	fprintf(stdout, "jumpin down \n\r");
	}
	else
	{
		switch(NTCmode)
		{
			case 1:  if(xfloat < 2.48) NTCmode = 2; break;
			case 2:  if(xfloat < 1.9)  NTCmode = 3; break;
			case 3:  if(xfloat < 2.15) NTCmode = 4; break;
			case 4:  if(xfloat < 2.18) NTCmode = 5; break;
		}
	}

	DDRA = (1<<(NTCmode+2));
	PORTA = 0;

	ADCSRA |= (1<<ADSC); //start another conversion
	
	return (T_NTC-273.15);
}

void initNTC()
{

	fprintf(stdout, "initing ntc");
	
	NTCmode = 1; //start in lowest gain mode
	
//	DDRA = (1<<3);
//	PORTA = ~(1<<3); //1111011;
	
	DDRA= (1<<(NTCmode+2));
	PORTA = 0x00;

	//set up ADC
	ADMUX = _BV(ADLAR) | _BV(REFS0);
	set_Ain_A0();
	_delay_ms(TIME_REFSETTLE_MS);
	ADCSRA |= ((1<<ADEN) | (1<<ADSC)) + 7;
}

/*

int main()
{
	uint16_t temperature;
	int t = 0;
	
	cli();
	initNTC();

	uart_init();
	stdout = stdin = stderr = &uart_str;
	fprintf(stdout,"Hellow PC!\n\r"); 


	sei();

	while(1)
	{
		temperature = readNTC();
		fprintf(stdout, "\n\rCurrent temperature is %u \n\r",temperature);
		if (t==0)
		{
			fprintf(stdout, "switching to A0 \n\r"); 
			set_Ain_A0();
			t = 1;
		}
		else if (t==1)
		{
			fprintf(stdout, "switching to A1 \n\r"); 
			set_Ain_A1();
			t = 0;
		}
		_delay_ms(1000);
		
	}
}

*/


