/*! 
*   @file stepper.c stepper control code source file
*   @author Adam Mendrela, Andrew Kerns
*/

#include "stepper.h"
#include <avr/io.h>

uint8_t enableStepper; //enable stepper
volatile uint8_t waitStepper;  //stepper timer
uint8_t stepper_stage;
int8_t steps;
uint8_t stepper_excited;

void initStepper(void)
{
	PORTD = (PORTD & 0x0F) | STEPPER_OFF;
}
	
//! drive stepper motor - non-blocking version - much more efficient than blocking version
int8_t setStepper(int8_t stepperstate, int8_t desired)
{
	int8_t target;
	/* input checking */
	if(desired > stepper_maxpos)
		target = stepper_maxpos;
	else if(desired < STEPPER_MINPOS)
		target = STEPPER_MINPOS;
	else
		target = desired;	
	/* compute number of 7.5 degree steps to take */
	steps += (target - stepperstate); //steps should be zero unless all steps weren't previously finished
	//fprintf(stdout,"max=%i,on=%i\n\r",stepper_maxpos,stepper_onpos);
	//fprintf(stdout,"sstate= %i, doing %i steps --> %i\n\r",sstate,steps,target);
	enableStepper = 1; 
	return target;
}

void fireCoil(int8_t* psteps, uint8_t* pstage, uint8_t* pexcited)
{
	//fprintf(stdout,"before firecoil: psteps=%i, pstage=%i \n\r",*psteps,*pstage);
	if (*psteps > 0)	//clockwise
	{
		if (*pexcited)
		{
			*pexcited = 0;
			(*psteps)--; 
			PORTD = (PORTD & 0x0F) | STEPPER_OFF;
		}
		else
		{
			*pexcited = 1;
			if (*pstage == 0)
				*pstage = 3;
			else
				(*pstage)--;
			switch(*pstage)
			{
				case 0:  PORTD = (PORTD & 0x0F) | STEPPER_STAGEA; break;
				case 1:  PORTD = (PORTD & 0x0F) | STEPPER_STAGEB; break;
				case 2:  PORTD = (PORTD & 0x0F) | STEPPER_STAGEC; break;
				case 3:  PORTD = (PORTD & 0x0F) | STEPPER_STAGED; break;
			}
		}
	}
	else if (*psteps < 0) //anti-clockwise
	{
		if (*pexcited)
		{
			*pexcited = 0;
			(*psteps)++; 
			PORTD = (PORTD & 0x0F) | STEPPER_OFF;
		}
		else
		{
			*pexcited = 1;
			if (*pstage == 3)
				*pstage = 0;
			else
				(*pstage)++;
			switch(*pstage)
			{
				case 0:  PORTD = (PORTD & 0x0F) | STEPPER_STAGEA; break;
				case 1:  PORTD = (PORTD & 0x0F) | STEPPER_STAGEB; break;
				case 2:  PORTD = (PORTD & 0x0F) | STEPPER_STAGEC; break;
				case 3:  PORTD = (PORTD & 0x0F) | STEPPER_STAGED; break;
			}
		}
	}
}


