/*! 
*   @file stepper.h stepper control code header file
*   @author Adam Mendrela, Andrew Kerns
*/

#ifndef __STEPPER_H_GUARD_
#define __STEPPER_H_GUARD_

#include "cook.h"
#include <inttypes.h>

//THESE MUST BE between 4 and 7, otherwise need to change which nibble is masked in the code
#define PIN_STEPPER_1A 4
#define PIN_STEPPER_1B 5
#define PIN_STEPPER_2A 6
#define PIN_STEPPER_2B 7

#define STEPPER_MINPOS	0

#define STEPPER_LOWPOWERMODE

//see <http://mechatronics.mech.northwestern.edu/design_ref/actuators/stepper_intro.html>
#ifdef STEPPER_LOWPOWERMODE
	#define STEPPER_STAGEA 0x90
	#define STEPPER_STAGEB 0xA0
	#define STEPPER_STAGEC 0x60
	#define STEPPER_STAGED 0x50
	#define STEPPER_OFF	   0xF0
#else
	#define STEPPER_STAGE1 0x90
	#define STEPPER_STAGE2 0x50
	#define STEPPER_STAGE3 0x60
	#define STEPPER_STAGE4 0xA0
#endif

#define WAIT_STEPPER_MS 9


extern uint8_t enableStepper; //enable stepper
extern volatile uint8_t waitStepper;  //stepper timer
extern uint8_t stepper_stage;
extern int8_t steps;
extern uint8_t stepper_excited;


//Function prototypes
int8_t setStepper(int8_t stepperstate, int8_t desired);
void fireCoil(int8_t* psteps, uint8_t* pstage, uint8_t* pexcited);
void initStepper(void);
inline void taskStepper();
inline void updateStepperTimer();

inline void taskStepper()
{
		if ((enableStepper==1) && (waitStepper==0))
		{
			fireCoil(&steps,&stepper_stage,&stepper_excited);
			if (steps==0) 
				enableStepper = 0;
			else
				waitStepper = WAIT_STEPPER_MS;
		}
}

inline void updateStepperTimer()
{
	if (waitStepper > 0)	  	--waitStepper;
}

#endif




