/*! 
*   @file userinterface.c keypad and LCD management source file
*   @author Adam Mendrela, Andrew Kerns
*/

#include "userinterface.h"

const uint8_t lcd_ptcont[] PROGMEM =    "Press # to continue \0";
const uint8_t lcd_ptcook[] PROGMEM =    "Press # to cook     \0";
const uint8_t lcd_clearline[] PROGMEM = "                    \0";
const uint8_t lcd_welcome[] PROGMEM =   "Welcome to Hornilla!\0";
const uint8_t lcd_warning1[] PROGMEM =  "WARNINGS:           \0";
const uint8_t lcd_warning2[] PROGMEM =  "Turn dial to OFF    \0";
const uint8_t lcd_warning3[] PROGMEM =  "Press * to escape   \0";
const uint8_t lcd_warning4[] PROGMEM =  "at any time.        \0";
const uint8_t lcd_home1[] PROGMEM =     "A) Quick autotemp   \0";
const uint8_t lcd_home2[] PROGMEM =     "B) DC heat          \0";
const uint8_t lcd_home3[] PROGMEM =     "C) Autocook         \0";
const uint8_t lcd_home4[] PROGMEM =     "D) Calibrate stepper\0";
const uint8_t lcd_autotemp1[] PROGMEM = "Enter temperature:  \0";
const uint8_t lcd_autotemp2[] PROGMEM = "A) Toggle F/C       \0";
const uint8_t lcd_dcheat11[] PROGMEM =  "Enter time:         \0";
const uint8_t lcd_dcheat12[] PROGMEM =  "  :                 \0";
const uint8_t lcd_dcheat21[] PROGMEM =  "1) Heat+\0";
const uint8_t lcd_dcheat22[] PROGMEM =  "2) Heat-            \0";
const uint8_t lcd_dcheat23[] PROGMEM =  " Low | Medium | High\0";
const uint8_t lcd_autocook_entry1[] PROGMEM = "A) Green Tea        \0";
const uint8_t lcd_autocook_entry2[] PROGMEM = "B) Medium Beef      \0";
const uint8_t lcd_autocook_entry3[] PROGMEM = "C) Soft-boiled Eggs \0";
const uint8_t lcd_cal11[] PROGMEM =     "Confirm dial is     \0";
const uint8_t lcd_cal12[] PROGMEM =     "completely OFF      \0";
const uint8_t lcd_cal21[] PROGMEM =     "Press 2 until dial  \0";
const uint8_t lcd_cal22[] PROGMEM =     "is at minimum ON    \0";
const uint8_t lcd_cal31[] PROGMEM =     "Press 2 until dial  \0";
const uint8_t lcd_cal32[] PROGMEM =     "has reached HIGH    \0";
const uint8_t lcd_cal33[] PROGMEM =     "Only then, press #  \0";
const uint8_t lcd_cal34[] PROGMEM =     "to exit calibration \0";

				 
char param_str[PARAM_NUMDIGITS+1];
uint8_t param_count = 0; 
uint16_t param_num;

uint8_t dbstate;

uint8_t key ;   // The raw keyscan
uint8_t butnum ;// The decoded button number
uint8_t keytbl[16]={0xee, 0xed, 0xeb, 0xe7, //key pad scan table
						  0xde, 0xdd, 0xdb, 0xd7, 
						  0xbe, 0xbd, 0xbb, 0xb7, 
						  0x7e, 0x7d, 0x7b, 0x77};
//uint8_t lcd_buffer[21];
uint8_t lcd_buffer[25];
uint8_t lcd_bar[21];

//! read the state of the keypad
/*! this gets called every TIME_KEY_MS milliseconds.
it must update keypressed 
*/
uint8_t scanKeypad()
{		
	//fprintf(stdout, "Scanning keypad...\n\r");
	//get lower nibble
  	DDRB = 0x0f;
  	PORTB = 0xf0; 
  	_delay_us(5);
  	key = PINB;
  	  
  	//get upper nibble
  	DDRB = 0xf0;
  	PORTB = 0x0f; 
  	_delay_us(5);
  	key = key | PINB;
  	  
  	//find matching keycode in keytbl
  	if (key != 0xff)
  	{  
  	  for (butnum=0; butnum<MAXKEYS; butnum++)
  	  {   
  	  	if (keytbl[butnum]==key)  break;   
  	  }
  	  if (butnum==MAXKEYS) 
		butnum=0;
  	  else 
		butnum++;	//adjust by one to make range 1-16
  	} 
  	else butnum=0;

  	//if(butnum != 0)
	//	fprintf(stdout, "b=%i\n\r", butnum); //just for debugging

	return updateDB(butnum);
}

//! Update Debouncing State Machine
/*! 
	@param keypressed	raw output of scanKeypad that needs to be debounced
	- update dbstate depending on keypressed timing
*/
uint8_t updateDB(uint8_t keypressed)
{
	//fprintf(stdout,"DBSTATE = %i \n\r",dbstate);
	switch(dbstate)
	{
		case DB_REL: 
			if (keypressed) 
				dbstate = DB_MPRESS; //first evidence of press
			break;
		case DB_MPRESS:
			if (keypressed) 
			{
				dbstate = DB_PRESS;
				
					//fprintf(stdout, "pressed %u \n\r", keypressed);
				
				return keypressed; //updateState(keypressed);
			}
			else
				dbstate = DB_REL; 
			break;
		case DB_PRESS:
			if (keypressed) 
				dbstate = DB_PRESS; 
			else
				dbstate = DB_MREL; 
			break;
		case DB_MREL:
			if (keypressed) 
				dbstate = DB_PRESS;
			else
				dbstate = DB_REL;
			break;	
	}
	return 0;
}

/* send output to the LCD depending on what mode the program is in */
void updateLCD()
{
	uint8_t numOfBars = 0;
	uint8_t barCount;
	switch(mode)
	{
		case MODE_WELCOME:
			CopyStringtoLCD(lcd_welcome,0,0);
			CopyStringtoLCD(lcd_clearline,0,1);
			CopyStringtoLCD(lcd_clearline,0,2);
			CopyStringtoLCD(lcd_ptcont,0,3);
			break;
		case MODE_WARNING:
			CopyStringtoLCD(lcd_warning1,0,0);
			CopyStringtoLCD(lcd_warning2,0,1);
			CopyStringtoLCD(lcd_warning3,0,2);
			CopyStringtoLCD(lcd_warning4,0,3);
			break;
		case MODE_HOME:
			CopyStringtoLCD(lcd_home1,0,0);
			CopyStringtoLCD(lcd_home2,0,1);
			CopyStringtoLCD(lcd_home3,0,2);
			CopyStringtoLCD(lcd_home4,0,3);
			break;
		case MODE_AUTOTEMP:
			CopyStringtoLCD(lcd_autotemp1,0,0);
			fprintf(stdout,"ForC = %i \n\r",ForC);
			if (ForC == FORC_C)
				sprintf(lcd_buffer, "%-4s C            ", param_str);
			else
				sprintf(lcd_buffer, "%-4s F            ", param_str);
			LCDGotoXY(0,1);
			LCDstring(lcd_buffer, strlen((char*)lcd_buffer));
			//second line gets updated in another function
			CopyStringtoLCD(lcd_autotemp2,0,2);
			CopyStringtoLCD(lcd_ptcook,0,3);
				 
			break;
		case MODE_DCHEAT1:
			CopyStringtoLCD(lcd_dcheat11,0,0);
			CopyStringtoLCD(lcd_clearline,0,2);
			CopyStringtoLCD(lcd_ptcont,0,3);
			break;
		case MODE_DCHEAT2:
			CopyStringtoLCD(lcd_dcheat21,0,0);
			sprintf(lcd_buffer,"  %02u:%02u\0",timer_min,timer_s);
			LCDGotoXY(8,0);
			LCDstring(lcd_buffer, (uint8_t)strlen((char*)lcd_buffer));
			CopyStringtoLCD(lcd_dcheat22,0,1);
			CopyStringtoLCD(lcd_dcheat23,0,2);

			numOfBars = (sstate*20)/(stepper_maxpos-STEPPER_MINPOS);
			
            for(barCount = 0; barCount < 20;barCount++)
				if(barCount < numOfBars)
					lcd_bar[barCount] = 255;
				else
					lcd_bar[barCount] = ' ';
 			lcd_bar[21] = '\0';

          
			LCDGotoXY(0,3);
			LCDstring(lcd_bar, (uint8_t)strlen((char*)lcd_bar));

			//fprintf(stdout,"sstate = %i \n\r",sstate);
			break;
		case MODE_AUTOCOOK:
			CopyStringtoLCD(lcd_autocook_entry1,0,0);
			//CopyStringtoLCD(lcd_autocook_entry2,0,1);
			//CopyStringtoLCD(lcd_autocook_entry3,0,2);
			CopyStringtoLCD(lcd_autocook_entry2,0,1);
			CopyStringtoLCD(lcd_autocook_entry3,0,2);
			CopyStringtoLCD(lcd_clearline,0,3);
			break;

		case MODE_COOK:
		    
			//line0:  T = ?
			if (ForC == FORC_C)     		
				sprintf(lcd_buffer, "   T = %-4iC       ", (int16_t)(T+0.5)); // print in C
			else
				sprintf(lcd_buffer, "   T = %-4uF       ", (int16_t)(T*9.0/5.0 + 32.5)); // convert to F and print
			LCDGotoXY(0,0);
			LCDstring(lcd_buffer, strlen((char*)lcd_buffer));			
			
			//line1:  Tdes = ?
			if (ForC == FORC_C)     		
				sprintf(lcd_buffer, "Tdes = %-4uC       ", (int16_t)(Td+0.5)); // print in C
			else
				sprintf(lcd_buffer, "Tdes = %-4uF       ", (int16_t)(Td*9.0/5.0 + 32.5)); // convert to F and print
			LCDGotoXY(0,1);
			LCDstring(lcd_buffer, strlen((char*)lcd_buffer));
			
			//line2:  NTC sensor or IC sensor, current position
			sprintf(lcd_buffer, "pos = %2u/%02u         ", sstate, max_sstate);
			LCDGotoXY(0,2);
			LCDstring(lcd_buffer, strlen((char*)lcd_buffer));
			
			//line 3:  QMAX (and maybe Q?)
			
			if (probe == PROBE_NTC)
				sprintf(lcd_buffer, "NTC   %02u:%02u         ",timer_min,timer_s);
			else
				sprintf(lcd_buffer, "IC    %02u:%02u         ",timer_min,timer_s);
			LCDGotoXY(0,3);
			LCDstring(lcd_buffer, strlen((char*)lcd_buffer));
			break;
		case MODE_CAL1:
			CopyStringtoLCD(lcd_cal11,0,0);
			CopyStringtoLCD(lcd_cal12,0,1);
			CopyStringtoLCD(lcd_clearline,0,2);
			CopyStringtoLCD(lcd_ptcont,0,3);
			break;
		case MODE_CAL2:
			CopyStringtoLCD(lcd_cal21,0,0);
			CopyStringtoLCD(lcd_cal22,0,1);
			CopyStringtoLCD(lcd_clearline,0,2);
			CopyStringtoLCD(lcd_ptcont,0,3);
			break;
		case MODE_CAL3:
			CopyStringtoLCD(lcd_cal31,0,0);
			CopyStringtoLCD(lcd_cal32,0,1);
			CopyStringtoLCD(lcd_cal33,0,2);
			CopyStringtoLCD(lcd_cal34,0,3);
			break;
	}
}

//! Process the incoming key press
/*! 
This function is called by with any non-zero result of updateDB() (i.e., on a transition into DB_PRESS)
@param keypressed	the encoding for the key that has be affirmatively pressed according to the debounce state machine
--- Entry modes ----------------------------------------------------------------------------------  
Welcome Screen:  "Welcome to Robo-Hornilla!","","","Press # to continue." stay until press # --> go to Warning screen
Warning Screen:  "Warning1: turn dial to OFF before beginning","Warning2: at any point hit star to turn off heat","",""
				 stay until press # --> go to Home Screen
Home Screen:	 "A) Quick autotemp","B) DC heat","C)Autocook","D)Cal" --> go to correct settings mode
-----------------------------------------------------------------------------------------------------------
--- Settings modes ---------------------

Autotemp screen: "Enter temperature (x): ___". print numbers as they type
				 "Hit A to toggle between F and C" --> toggle x
				 "Press # to cook" --> go to cook mode
DC heat screen:  "1) Reduce heat","2) Increase heat"," Low | Medium | High",heat bar
					--> simply call setStepper(0,1) or (1,0)
Autocook screen: "A) use preset","B) define own settings" --> go to autocook-preset or autocook-custom
Autocook-preset: "A) boil water","B) 108F yeast water", "C) deep-frying canola oil","D) med-rare beef" --> go to cook
Autocook-custom1: "TBD"

--- Cook mode ---------------------------
Cook screen:  "T = {xxx.x} deg{ForC}","Tdes = {xxx.x} deg{ForC}","p = {NTC,IC}, Q = {xxxxx} W","Qlim = {xxxx or Inf}"
-----------------------------------------
--- Cal mode ----------------------------
Cal1 screen:  "Confirm dial is","completely OFF","","Press # to continue" --> press # to go to cal2
Cal2 screen:  "Press 2 until dial","is at minimum ON pos","","Press # to continue" --> press # to go to cal3
Cal3 screen:  "Press 2 until dial","has reached HIGH pos","Only then, press #","to exit calibration" --> pressing 2 calls stepper(curr,curr+1)
				--> pressing # to go to home screen

*/
void updateState(uint8_t keypressed)
{
	
	if (keypressed == KEY_STAR) // escape key. turn off cooker and re-start program (effectively)
	{
		// execute safety stop
		setStepper(stepper_maxpos,STEPPER_MINPOS); //assume worst case
		// reset menu tree
		mode = MODE_WELCOME;
		return;
	}

	switch(mode)
	{
		case MODE_WELCOME:
			if (keypressed == KEY_POUND)
			{
				mode = MODE_WARNING;
			}
			break;
		case MODE_WARNING:
			if (keypressed == KEY_POUND)
			{
				mode = MODE_HOME;
				//setStepper(4,0);
			}
			break;
		case MODE_HOME:
			// soft reset for userinterface.c
			param_num = 0;
			param_count = 0;
			for(uint8_t i=0; i < (PARAM_NUMDIGITS+1); i++)
				param_str[i] = 0; //clear the whole character array	
			//soft reset for the rest of the program
			softReset();
			switch(keypressed)
			{
				case KEY_A:
					mode = MODE_AUTOTEMP;
					break;
				case KEY_B:
					mode = MODE_DCHEAT1;
					CopyStringtoLCD(lcd_clearline,0,1);
					break;
				case KEY_C:
					mode = MODE_AUTOCOOK;
					break;
				case KEY_D:
					mode = MODE_CAL1;
					break;
			}
			break;
		case MODE_AUTOTEMP:
			switch(keypressed)
			{
				// A toggles F or C
				case KEY_A:
					if (ForC == FORC_C)
						ForC = FORC_F;
					else
						ForC = FORC_C;
					break;
				// keys to handle numeric input and KEY_POUND to process the input
				case KEY_0:
					param_str[param_count] = 0x30; // ACSCII encoding of '0' is 0x30
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_1:
					param_str[param_count] = 0x31; // ACSCII encoding of '1' is 0x31
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_2:
					param_str[param_count] = 0x32; // ACSCII encoding of '2' is 0x32
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_3:
					param_str[param_count] = 0x33; // ACSCII encoding of '3' is 0x33
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_4:
					param_str[param_count] = 0x34; // ACSCII encoding of '4' is 0x34
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_5:
					param_str[param_count] = 0x35; // ACSCII encoding of '5' is 0x35
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_6:
					param_str[param_count] = 0x36; // ACSCII encoding of '6' is 0x36
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_7:
					param_str[param_count] = 0x37; // ACSCII encoding of '7' is 0x37
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_8:
					param_str[param_count] = 0x38; // ACSCII encoding of '8' is 0x38
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_9:
					param_str[param_count] = 0x39; // ACSCII encoding of '9' is 0x39
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_POUND:
					if (ForC == FORC_C)
						Td = (float)param_num; // keep in C
					else
						Td = ((float)param_num-32)*5/9; // convert to C
					param_num = 0;
					param_count = 0;
					for(uint8_t i=0; i < (PARAM_NUMDIGITS+1); i++)
						param_str[i] = 0; //clear the whole character array	
						
					//setting cook mode variables
					max_sstate = stepper_maxpos;	
					Td_deadzone = -1000.0; //do this so that the deadzone is not used in autotemp mode
					setTimerMinSec(99,99);
					timer_on = 0;
					mode = MODE_COOK;
					selectProbe();
					break;
			}
			if (ForC == FORC_C)
				sprintf(lcd_buffer, "%-4s %c            ",param_str,'C');
			else
				sprintf(lcd_buffer, "%-4s %c            ",param_str,'F');
			LCDGotoXY(0,1);
			LCDstring(lcd_buffer, strlen((char*)lcd_buffer));
			break;
		case MODE_DCHEAT1:
			switch(keypressed)
			{
				// keys to handle numeric input and KEY_POUND to process the input
				case KEY_0:
					param_str[param_count] = 0x30; // ACSCII encoding of '0' is 0x30
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_1:
					param_str[param_count] = 0x31; // ACSCII encoding of '1' is 0x31
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_2:
					param_str[param_count] = 0x32; // ACSCII encoding of '2' is 0x32
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_3:
					param_str[param_count] = 0x33; // ACSCII encoding of '3' is 0x33
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_4:
					param_str[param_count] = 0x34; // ACSCII encoding of '4' is 0x34
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_5:
					param_str[param_count] = 0x35; // ACSCII encoding of '5' is 0x35
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_6:
					param_str[param_count] = 0x36; // ACSCII encoding of '6' is 0x36
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_7:
					param_str[param_count] = 0x37; // ACSCII encoding of '7' is 0x37
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_8:
					param_str[param_count] = 0x38; // ACSCII encoding of '8' is 0x38
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_9:
					param_str[param_count] = 0x39; // ACSCII encoding of '9' is 0x39
					param_count++;
					param_num = atoi(param_str);
					break;
				case KEY_POUND:
					setTimerMinSec(param_num/100,param_num%100);
					fprintf(stdout,"minutes = %u, seconds = %u \n\r",timer_min,timer_s);
					param_num = 0;
					param_count = 0;
					for(uint8_t i=0; i < (PARAM_NUMDIGITS+1); i++)
						param_str[i] = 0; //clear the whole character array	
					mode = MODE_DCHEAT2;
					selectProbe();
					break;
			}
			/*
			if (param_num == 0)
				sprintf(lcd_buffer, "00:00               ");
			else if (param_num < 10)
				sprintf(lcd_buffer, "00:0%1u               ",param_num);
			else if (param_num < 100)
				sprintf(lcd_buffer, "00:%2u               ",param_num);
			else if (param_num < 1000)
				sprintf(lcd_buffer, "0%1u:%2u               ",param_num/100,param_num%100);
			else
			*/
				sprintf(lcd_buffer, "%02u:%02u               ",param_num/100,param_num%100);

			LCDGotoXY(0,1);
			LCDstring(lcd_buffer, strlen((char*)lcd_buffer));
			break;
		case MODE_DCHEAT2:
			switch(keypressed)
			{
				case KEY_1:
					sstate = setStepper(sstate,(sstate-1));
					break;
				case KEY_2:
					sstate = setStepper(sstate,(sstate+1));
					break;
				case KEY_4:
					sstate = setStepper(sstate,(sstate-4));
					break;
				case KEY_5:
					sstate = setStepper(sstate,(sstate+4));
					break;
				case KEY_7:
					sstate = setStepper(sstate,(sstate-10));
					break;
				case KEY_8:
					sstate = setStepper(sstate,(sstate+10));
					break;
			}
			break;
		case MODE_AUTOCOOK:
			switch(keypressed)
			{
				case KEY_A:
					//Green Tea
					Td = 68;
					Td_deadzone = 4.0;
					max_sstate = stepper_maxpos;
					setTimerMinSec(2,00);
					timer_on = 0;
					mode = MODE_COOK;
					selectProbe();
					break;
				case KEY_B:
					//Medium Beef TODO
					Td = 62;
					Td_deadzone = 0.1;
					max_sstate = stepper_maxpos*3/4;
					setTimerMinSec(0,0);
					timer_on = 0;
					mode = MODE_COOK;
					selectProbe();
					break;
				case KEY_C:
					//set Tdes, Qmax, etc.
					//Soft-boiled Eggs TODO
					Td = 63;
					Td_deadzone = 3.0;
					max_sstate = stepper_maxpos;
					setTimerMinSec(45,0);
					timer_on = 0;
					mode = MODE_COOK;
					selectProbe();
					break;
			}
			break;
		case MODE_COOK:
			//don't need to do anything
			break;
		case MODE_CAL1:
			if (keypressed == KEY_POUND)
			{
				stepper_maxpos = 125;
				mode = MODE_CAL2;
			}
			break;
		case MODE_CAL2:
			fprintf(stdout, "sstate=%i,max=%i\n\r",sstate, stepper_maxpos);
			switch(keypressed)
			{
				case KEY_POUND:
					// set stepper_maxpos to the current sstate
					stepper_onpos = sstate;
					eeprom_write_byte((uint8_t*)EEPROM_BYTE_ONPOS, stepper_onpos);
					mode = MODE_CAL3;
					break;
				case KEY_2:
					sstate = setStepper(sstate,(sstate+1));
					break;
				case KEY_5:
					sstate = setStepper(sstate,(sstate+4));
					break;
				case KEY_8:
					sstate = setStepper(sstate,(sstate+10));
					break;
			}
			break;
		case MODE_CAL3:
			fprintf(stdout,"cal3,kp=%u",keypressed);
			switch(keypressed)
			{
				case KEY_POUND:
					// set stepper_maxpos to the current sstate
					stepper_maxpos = sstate;
					eeprom_write_byte((uint8_t*)EEPROM_BYTE_MAXPOS, stepper_maxpos);
					// execute safety stop
					setStepper(stepper_maxpos,STEPPER_MINPOS); //assume worst case
					// reset menu tree
					mode = MODE_HOME;
					break;
				case KEY_2:
					sstate = setStepper(sstate,(sstate+1));
					break;
				case KEY_5:
					sstate = setStepper(sstate,(sstate+4));
					break;
				case KEY_8:
					sstate = setStepper(sstate,(sstate+10));
					break;
			}
			break;
	}
}
